package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type GridStoreLogApi struct {
}

// CreateGridStoreLog
// @Tags      日志管理
// @Summary   创建存物日志(测试使用)
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    model.GridStoreLog true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/store [post]
func (api *GridStoreLogApi) CreateGridStoreLog(c *gin.Context) {
	var log model.GridStoreLog
	if err := bind.Bind(c, &log, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := gridStoreLogService.CreateGridStoreLog(log)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteGridStoreLogByIds
// @Tags      日志管理
// @Summary   批量删除存物日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/store [delete]
func (api *GridStoreLogApi) DeleteGridStoreLogByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = gridStoreLogService.DeleteGridStoreLogByIds(IDS)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetGridStoreLogList
// @Tags      日志管理
// @Summary   分页获取存物日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridStoreLogSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/store/list [get]
func (api *GridStoreLogApi) GetGridStoreLogList(c *gin.Context) {
	var pageInfo request.GridStoreLogSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := gridStoreLogService.GetGridStoreLogList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}

// GetGridStoreLogStatistics
// @Tags      日志管理
// @Summary   存物统计
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridStoreLogStatistics true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/store/statistics [get]
func (api *GridStoreLogApi) GetGridStoreLogStatistics(c *gin.Context) {
	var pageInfo request.GridStoreLogStatistics
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, err := gridStoreLogService.GetGridStoreLogStatistics(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List: list,
	})
	return
}
