package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type GridWarnLogApi struct {
}

// CreateGridWarnLog
// @Tags      日志管理
// @Summary   创建警告日志(测试使用)
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    model.GridWarnLog true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/warn [post]
func (api *GridWarnLogApi) CreateGridWarnLog(c *gin.Context) {
	var log model.GridWarnLog
	if err := bind.Bind(c, &log, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := gridWarnLogService.CreateGridWarnLog(log)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteGridWarnLogByIds
// @Tags      日志管理
// @Summary   批量删除开警告日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/warn [delete]
func (api *GridWarnLogApi) DeleteGridWarnLogByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = gridWarnLogService.DeleteGridWarnLogByIds(IDS)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetGridWarnLogList
// @Tags      日志管理
// @Summary   分页获取警告日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridStoreLogSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/warn/list [get]
func (api *GridWarnLogApi) GetGridWarnLogList(c *gin.Context) {
	var pageInfo request.GridWarnLogSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := gridWarnLogService.GetGridWarnLogList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}

// GetGridWarnLogStatistics
// @Tags      日志管理
// @Summary   告警统计
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.GridWarnLogStatistics true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/warn/statistics [get]
func (api *GridWarnLogApi) GetGridWarnLogStatistics(c *gin.Context) {
	var pageInfo request.GridWarnLogStatistics
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, err := gridWarnLogService.GetGridWarnLogStatistics(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List: list,
	})
	return
}
