package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/bus/model/response"
	"dy-admin/internal/pcmserver/bus/service"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"

	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type LeaveLogApi struct {
}

// CreateLeaveLog
// @Tags      请假管理
// @Summary   创建请假申请
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.LeaveLog true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/leave [post]
func (lla *LeaveLogApi) CreateLeaveLog(c *gin.Context) {
	var LeaveLog model.LeaveLog
	if err := bind.Bind(c, &LeaveLog, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(LeaveLog, utils.LeaveLogVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	LeaveLog.CreateBy = global.JWT.GetUserID(c)
	err = leaveLogService.CreateLeaveLog(LeaveLog)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateLeaveLog
// @Tags      请假管理
// @Summary   更新请假
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.LeaveLog true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/leave [put]
func (lla *LeaveLogApi) UpdateLeaveLog(c *gin.Context) {
	var LeaveLog model.LeaveLog
	if err := bind.Bind(c, &LeaveLog, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(LeaveLog.Model, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = utils.Verify(LeaveLog, utils.LeaveLogVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	LeaveLog.UpdateBy = global.JWT.GetUserID(c)
	err = leaveLogService.UpdateLeaveLog(LeaveLog)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteLeaveLogByIds
// @Tags      请假管理
// @Summary   删除请假
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/leave [delete]
func (lla *LeaveLogApi) DeleteLeaveLogByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = leaveLogService.DeleteLeaveLogByIds(IDS)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// GetLeaveLog
// @Tags      请假管理
// @Summary   获取请假申请
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/leave [get]
func (lla *LeaveLogApi) GetLeaveLog(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	log, err := leaveLogService.GetLeaveLog(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.LeaveLogResponse{LeaveLog: log})
	return
}

// GetLeaveLogList
// @Tags      请假管理
// @Summary   分页获取请假
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.LeaveLogRecordSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/leave/list [get]
func (lla *LeaveLogApi) GetLeaveLogList(c *gin.Context) {
	var pageInfo request.LeaveLogRecordSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	dataPermission, err := service.NewBusPermission(c)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := leaveLogService.GetLeaveLogList(pageInfo, dataPermission)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}
