package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type SpecialTimeApi struct {
}

// CreateSpecialTime
// @Tags      例外时间管理
// @Summary   创建例外时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.SpecialTime true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/specTime [post]
func (api *SpecialTimeApi) CreateSpecialTime(c *gin.Context) {
	var specTime model.SpecialTime
	if err := bind.Bind(c, &specTime, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(specTime, utils.SpecialTimeVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	specTime.CreateBy = global.JWT.GetUserID(c)
	err = specialTimeService.CreateSpecialTime(specTime)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateSpecialTime
// @Tags      例外时间管理
// @Summary   更新例外时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.SpecialTime true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/specTime [put]
func (api *SpecialTimeApi) UpdateSpecialTime(c *gin.Context) {
	var specTime model.SpecialTime
	if err := bind.Bind(c, &specTime, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(specTime.Model, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	specTime.UpdateBy = global.JWT.GetUserID(c)
	err = specialTimeService.UpdateSpecialTime(specTime)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateSpecialTimeStatus
// @Tags      例外时间管理
// @Summary   更新例外时间状态
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.UpdateSpecialTimeStatusRequest true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/specTime/status [put]
func (api *SpecialTimeApi) UpdateSpecialTimeStatus(c *gin.Context) {
	var req request.UpdateSpecialTimeStatusRequest
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = specialTimeService.UpdateSpecialTimeStatus(req.ID, req.Status)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteSpecialTime
// @Tags      例外时间管理
// @Summary   删除例外时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/specTime [delete]
func (api *SpecialTimeApi) DeleteSpecialTime(c *gin.Context) {
	var ID common.IdReq
	if err := bind.Bind(c, &ID, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ID, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = specialTimeService.DeleteSpecialTime(ID.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetSpecialTimeList
// @Tags      例外时间管理
// @Summary   分页获取例外时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body    request.SpecialTimeSearch true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/specTime [get]
func (api *SpecialTimeApi) GetSpecialTimeList(c *gin.Context) {
	var pageInfo request.SpecialTimeSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, total, err := specialTimeService.GetSpecialTimeList(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}
