package v1

import (
	"dy-admin/internal/pcmserver/bus/model"
	"dy-admin/internal/pcmserver/bus/model/request"
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

type WorkTimeApi struct {
}

// CreateWorkTime
// @Tags      工作时间管理
// @Summary   创建工作时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.WorkTime true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/workTime [post]
func (api *WorkTimeApi) CreateWorkTime(c *gin.Context) {
	var workTime model.WorkTime
	if err := bind.Bind(c, &workTime, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(workTime, utils.WorkTimeVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	workTime.CreateBy = global.JWT.GetUserID(c)
	err = workTimeService.CreateWorkTime(workTime)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateWorkTime
// @Tags      工作时间管理
// @Summary   更新工作时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.WorkTime true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/workTime [put]
func (api *WorkTimeApi) UpdateWorkTime(c *gin.Context) {
	var workTime model.WorkTime
	if err := bind.Bind(c, &workTime, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(workTime.Model, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	workTime.UpdateBy = global.JWT.GetUserID(c)
	err = workTimeService.UpdateWorkTime(workTime)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// UpdateWorkTimeStatus
// @Tags      工作时间管理
// @Summary   更新状态
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     request.UpdateWorkTimeStatusRequest true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/workTime/status [put]
func (api *WorkTimeApi) UpdateWorkTimeStatus(c *gin.Context) {
	var req request.UpdateWorkTimeStatusRequest
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = workTimeService.UpdateWorkTimeStatus(req.ID, req.Status)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// DeleteWorkTime
// @Tags      工作时间管理
// @Summary   删除工作时间
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/workTime [delete]
func (api *WorkTimeApi) DeleteWorkTime(c *gin.Context) {
	var ID common.IdReq
	if err := bind.Bind(c, &ID, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(ID, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = workTimeService.DeleteWorkTime(ID.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetWorkTimeList
// @Tags      工作时间管理
// @Summary   获取全部设置的工作列表
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Success   200   {object}  rescode.ErrCode
// @Router    /bus/v1/workTime [get]
func (api *WorkTimeApi) GetWorkTimeList(c *gin.Context) {
	list, err := workTimeService.GetWorkTimeList()
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List: list,
	})
	return
}
