package service

import (
	"dy-admin/pkg/log"
	"go.uber.org/zap"
	"sync"
)

type HomeClientHub struct {
	clients          map[string]*HomeClient
	mutex            sync.Mutex
	clientRegister   chan *HomeClient
	clientUnRegister chan *HomeClient
	broadcast        chan *HomeMsg
}

var homeClientHub *HomeClientHub

func initHomeClientHub() {
	homeClientHub = newHomeClientHub()
	go homeClientHub.run()
}

func newHomeClientHub() *HomeClientHub {
	return &HomeClientHub{
		clients:          make(map[string]*HomeClient),
		mutex:            sync.Mutex{},
		clientRegister:   make(chan *HomeClient),
		clientUnRegister: make(chan *HomeClient),
		broadcast:        make(chan *HomeMsg, 2048),
	}
}

func destroyHomeClientHub() {
	for _, client := range homeClientHub.clients {
		if client != nil {
			_ = client.clientConn.Close()
			close(client.send)
		}
	}
	close(homeClientHub.clientRegister)
	close(homeClientHub.clientUnRegister)
	homeClientHub = nil
}

func (hch *HomeClientHub) run() {
	for {
		select {
		case homeClient := <-hch.clientRegister:
			if homeClient == nil {
				continue
			}
			hch.mutex.Lock()
			if _, ok := hch.clients[homeClient.address]; !ok {
				hch.clients[homeClient.address] = homeClient
			}
			hch.mutex.Unlock()
			log.Info("client register success", zap.String("address", homeClient.address))
		case homeClient := <-hch.clientUnRegister:
			if homeClient == nil {
				continue
			}
			hch.mutex.Lock()
			if _, ok := hch.clients[homeClient.address]; ok {
				delete(hch.clients, homeClient.address)
			}
			hch.mutex.Unlock()
			log.Info("client unRegister success", zap.String("address", homeClient.address))
		case msg := <-hch.broadcast:
			for address, client := range hch.clients {
				// 判断当前数据是哪个部门的数据，只发给受影响的连接。不要全部发出去
				if msg.MsgType != 0 {
					_, ok := client.deptWithSon[msg.DeptID]
					if !ok {
						continue
					}
				}
				select {
				case client.send <- msg:
				default:
					log.Warn("client send channel is full", zap.String("address", address))
				}
			}
		}
	}
}
