package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/model/request"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"strings"
)

type OperationRecordApi struct{}

// CreateSysOperationRecord
// @Tags      操作日志管理
// @Summary   创建日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     model.OperationRecord true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/operation [post]
func (s *OperationRecordApi) CreateSysOperationRecord(c *gin.Context) {
	var sysOperationRecord model.OperationRecord
	if err := bind.Bind(c, &sysOperationRecord, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	sysOperationRecord.CreateBy = global.JWT.GetUserID(c)
	err := operationRecordService.CreateSysOperationRecord(sysOperationRecord)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, err)
	return
}

// DeleteSysOperationRecord
// @Tags      操作日志管理
// @Summary   删除单条日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/operation [delete]
func (s *OperationRecordApi) DeleteSysOperationRecord(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	err = operationRecordService.DeleteSysOperationRecord(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// DeleteSysOperationRecordByIds
// @Tags      操作日志管理
// @Summary   批量删除日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdsReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/operation/ids [delete]
func (s *OperationRecordApi) DeleteSysOperationRecordByIds(c *gin.Context) {
	var IDS common.IdsReq
	if err := bind.Bind(c, &IDS, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(IDS, utils.IdsVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	ids := strings.Split(IDS.IDs, ",")

	err = operationRecordService.DeleteSysOperationRecordByIds(ids)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
	return
}

// FindSysOperationRecord
// @Tags      操作日志管理
// @Summary   获取单条日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body     common.IdReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/operation [get]
func (s *OperationRecordApi) FindSysOperationRecord(c *gin.Context) {
	var idReq common.IdReq
	if err := bind.Bind(c, &idReq, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(idReq, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	reSysOperationRecord, err := operationRecordService.GetSysOperationRecord(idReq.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.OperationRecordResponse{OperationRecord: reSysOperationRecord})
	return
}

// GetSysOperationRecordList
// @Tags      操作日志管理
// @Summary   分页获取日志
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.OperationRecordSearch  true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/operation/list [get]
func (s *OperationRecordApi) GetSysOperationRecordList(c *gin.Context) {
	var pageInfo request.OperationRecordSearch
	if err := bind.Bind(c, &pageInfo, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	list, total, err := operationRecordService.GetSysOperationRecordInfoList(pageInfo)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     pageInfo.Page,
		PageSize: pageInfo.PageSize,
	})
	return
}
