package v1

import (
	"dy-admin/internal/pcmserver/common"
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/internal/pcmserver/sys/model/request"
	"dy-admin/internal/pcmserver/sys/model/response"
	"dy-admin/internal/pcmserver/utils"
	"dy-admin/pkg/bind"
	"dy-admin/pkg/jwt"
	"dy-admin/pkg/log"
	"dy-admin/pkg/rescode"
	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
	"go.uber.org/zap"
)

type UserApi struct {
}

// Login
// @Tags     系统用户管理
// @Summary  用户登录
// @Produce   application/json
// @Param    data  body      request.Login true  "参数"
// @Success  200   {object}  rescode.ErrCode
// @Router   /base/login [post]
func (s *UserApi) Login(c *gin.Context) {
	var logErr error
	var req request.Login
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		logErr = err
		rescode.WriteErrResponse(c, err)
		return
	}

	defer func() {
		// 记录登录日志
		err := loginRecordService.CreateLoginRecord(c, req.Username, logErr)
		if err != nil {
			log.Error("err:", zap.Error(err))
		}
	}()

	err := utils.Verify(req, utils.LoginVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		logErr = err
		rescode.WriteErrResponse(c, err)
		return
	}
	user, err := userService.Login(model.User{Username: req.Username, Password: req.Password})
	if err != nil {
		logErr = err
		rescode.WriteErrResponse(c, err)

		return
	}
	token, err := s.TokenNext(c, user)
	if err != nil {
		logErr = err
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.LoginResponse{
		User:  user,
		Token: token,
	})
	return
}

func (s *UserApi) TokenNext(c *gin.Context, user model.User) (string, error) {
	j := global.JWT
	claims := j.CreateClaims(jwt.BaseClaims{
		UUID:     user.UUID,
		UserID:   user.UserID,
		NickName: user.NickName,
		Username: user.Username,
		RoleIDs:  user.RoleIds,
		DeptID:   user.DeptId,
	})
	token, err := j.CreateToken(claims)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return "", err
	}

	return token, nil
}

// Register
// @Tags     系统用户管理
// @Summary  用户注册
// @Security  ApiKeyAuth
// @Produce   application/json
// @Param    data  body      request.Register true  "参数"
// @Success  200   {object}  rescode.ErrCode
// @Router   /sys/v1/user/register [post]
func (s *UserApi) Register(c *gin.Context) {
	var req request.Register
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.RegisterVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	roleIds := make([]int, 0)

	if req.RoleIds == 0 {
		// 2是一个默认角色。在数据初始化时完成创建。每个角色都有一些默认权限是必给的。我们使用默认角色这样一个角色来将这些必给权限赋予用户。
		// 当用户更换角色时，默认角色自动删除。这些权限在每个角色都有，所以这些必给的权限依然会有效。
		roleIds = append(roleIds, 2)

	} else {
		roleIds = append(roleIds, req.RoleIds)
	}

	user := model.User{
		Username: req.Username,
		Password: req.Password,
		NickName: req.NickName,
		Phone:    req.Phone,
		Salt:     "",
		Avatar:   "",
		Sex:      req.Sex,
		Email:    req.Email,
		DeptId:   req.DeptId,
		Remark:   req.Remark,
		Status:   req.Status,
		RoleIds:  roleIds,
	}

	user.CreateBy = global.JWT.GetUserID(c)
	u, err := userService.Register(user)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, response.SysUserResponse{User: u})
}

// ChangePassword
// @Tags     系统用户管理
// @Summary  修改密码
// @Security  ApiKeyAuth
// @Produce  application/json
// @Param     data  body      request.ChangePasswordReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/user/changePassword [put]
func (s *UserApi) ChangePassword(c *gin.Context) {
	var req request.ChangePasswordReq
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.ChangePasswordVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	userId := global.JWT.GetUserID(c)
	err = userService.ChangePassword(model.User{UserID: userId, Password: req.Password}, req.NewPassword)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetUserList
// @Tags     系统用户管理
// @Summary  用户列表
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.SearchUserReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/user/list [get]
func (s *UserApi) GetUserList(c *gin.Context) {
	var req request.SearchUserReq
	if err := bind.Bind(c, &req, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	list, total, err := userService.GetUserInfoList(req)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, common.PageResult{
		List:     list,
		Total:    total,
		Page:     req.Page,
		PageSize: req.PageSize,
	})
}

// DeleteUser
// @Tags     系统用户管理
// @Summary  删除用户
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.GetById true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/user [delete]
func (s *UserApi) DeleteUser(c *gin.Context) {
	var reqId common.GetById
	if err := bind.Bind(c, &reqId, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(reqId, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	userId := global.JWT.GetUserID(c)
	if userId == reqId.ID {
		err = rescode.RegisterErrorWithCause(nil, code.ErrCodeMap[code.ErrKillSelfCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = userService.DeleteUser(reqId.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// SetUserInfo
// @Tags     系统用户管理
// @Summary  更新用户信息
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.ChangeUserInfoReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/user/setUserInfo [put]
func (s *UserApi) SetUserInfo(c *gin.Context) {
	var req request.ChangeUserInfoReq
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	err := utils.Verify(req, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	roleIds := make([]int, 0)

	if req.RoleIds == 0 {
		// 2是一个默认角色。在数据初始化时完成创建。每个角色都有一些默认权限是必给的。我们使用默认角色这样一个角色来将这些必给权限赋予用户。
		// 当用户更换角色时，默认角色自动删除。这些权限在每个角色都有，所以这些必给的权限依然会有效。
		roleIds = append(roleIds, 2)

	} else {
		roleIds = append(roleIds, req.RoleIds)
	}

	err = userService.UpdateUser(model.User{
		UserID:   req.UserId,
		NickName: req.NickName,
		Phone:    req.Phone,
		Username: req.Username,
		Salt:     "",
		Avatar:   "",
		Sex:      req.Sex,
		Email:    req.Email,
		DeptId:   req.DeptId,
		Remark:   req.Remark,
		Status:   req.Status,
		RoleIds:  roleIds,
	})
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// SetSelfInfo
// @Tags     系统用户管理
// @Summary  更新自己信息
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      request.ChangeSelfInfoReq true  "参数"
// @Success   200   {object}  rescode.ErrCode
// @Router    /sys/v1/user/setSelfInfo [put]
func (s *UserApi) SetSelfInfo(c *gin.Context) {
	var req request.ChangeSelfInfoReq
	if err := bind.Bind(c, &req, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	userId := global.JWT.GetUserID(c)
	roleIds := global.JWT.GetUserRoleIDs(c)
	err := userService.UpdateUser(model.User{
		UserID:   userId,
		NickName: req.NickName,
		Phone:    req.Phone,
		Salt:     "",
		Avatar:   "",
		Sex:      req.Sex,
		Email:    req.Email,
		DeptId:   req.DeptId,
		Remark:   req.Remark,
		Status:   req.Status,
		RoleIds:  roleIds,
	})
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}

// GetUserInfo
// @Tags     系统用户管理
// @Summary   获取用户
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.GetById true  "参数"
// @Success   200  {object}  rescode.ErrCode
// @Router    /sys/v1/user/getUserInfo [get]
func (s *UserApi) GetUserInfo(c *gin.Context) {
	var reqId common.GetById
	if err := bind.Bind(c, &reqId, binding.Query); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(reqId, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}

	user, err := userService.GetUserByID(reqId.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.SysUserResponse{User: user})
}

// GetInfo
// @Tags     系统用户管理
// @Summary   token获取用户信息
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Success   200  {object}  rescode.ErrCode
// @Router    /sys/v1/user/getInfo [get]
func (s *UserApi) GetInfo(c *gin.Context) {
	userId := global.JWT.GetUserID(c)
	user, err := userService.GetUserByID(userId)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	rescode.WriteSuccessResponse(c, response.SysUserResponse{User: user})
}

// ResetPassword
// @Tags     系统用户管理
// @Summary   重置用户密码为123456
// @Security  ApiKeyAuth
// @accept    application/json
// @Produce   application/json
// @Param     data  body      common.GetById true  "参数"
// @Success   200  {object}  rescode.ErrCode
// @Router    /sys/v1/user/resetPassword [put]
func (s *UserApi) ResetPassword(c *gin.Context) {
	var reqId common.GetById
	if err := bind.Bind(c, &reqId, binding.JSON); err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}

	err := utils.Verify(reqId, utils.IdVerify)
	if err != nil {
		err = rescode.RegisterErrorWithMessage(err, err.Error(), code.ErrCodeMap[code.ErrValidationCode])
		rescode.WriteErrResponse(c, err)
		return
	}
	err = userService.ResetPassword(reqId.ID)
	if err != nil {
		rescode.WriteErrResponse(c, err)
		return
	}
	rescode.WriteSuccessResponse(c, nil)
}
