package service

import (
	"dy-admin/internal/pcmserver/global"
	"dy-admin/internal/pcmserver/pkg/code"
	"dy-admin/internal/pcmserver/sys/model"
	"dy-admin/pkg/rescode"
	"gorm.io/gorm"
	"strconv"
)

type DeptService struct {
}

func (ds *DeptService) GetSonDeptIds(deptId int) (ids []int, err error) {
	var depts []model.Dept
	err = global.DB.Where("dept_path like ?", "%"+strconv.Itoa(deptId)+"%").Order("sort asc").Find(&depts).Error
	if err != nil {
		return ids, rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	for _, dept := range depts {
		ids = append(ids, dept.DeptID)
	}
	return
}

func (ds *DeptService) GetDeptById(deptId int) (dept model.Dept, err error) {

	treeMap, err := ds.getDeptTree()
	if err != nil {
		return
	}
	err = global.DB.First(&dept, deptId).Error
	if err != nil && err != gorm.ErrRecordNotFound {
		return dept, rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	err = ds.getChildrenList(&dept, treeMap)
	if err != nil {
		return
	}
	return
}

func (ds *DeptService) CreateDept(dept model.Dept) (err error) {

	tx := global.DB.Begin()
	defer func() {
		if err != nil {
			tx.Rollback()
		} else {
			tx.Commit()
		}
	}()

	err = global.DB.Create(&dept).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}

	deptPath := strconv.Itoa(dept.DeptID) + "/"
	if dept.ParentId != 0 {
		var deptP model.Dept
		err = global.DB.First(&deptP, dept.ParentId).Error
		if err != nil {
			return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
		}
		deptPath = deptP.DeptPath + deptPath
	} else {
		deptPath = "/0/" + deptPath
	}

	if err = tx.Model(&dept).Update("dept_path", deptPath).Error; err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}

	return
}

func (ds *DeptService) UpdateDept(dept model.Dept) (err error) {
	deptPath := strconv.Itoa(dept.DeptID) + "/"
	if dept.ParentId != 0 {
		var deptP model.Dept
		err = global.DB.First(&deptP, dept.ParentId).Error
		if err != nil {
			return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
		}
		dept.DeptPath = deptP.DeptPath + deptPath
	} else {
		dept.DeptPath = "/0/" + deptPath
	}

	err = global.DB.Updates(dept).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	return
}

func (ds *DeptService) DeleteDept(deptId int) (err error) {
	sonIds, err := ds.GetSonDeptIds(deptId)
	if len(sonIds) > 1 {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ERRDeptHasSonCode])
	}

	// 如果部门存在人员，不允许删除
	_, count, err := ServicesGroupApp.UserService.GetUserInfoByDeptID(deptId)
	if err != nil {
		return err
	}
	if count != 0 {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDeptHasUserCode])
	}

	err = global.DB.Delete(&model.Dept{}, deptId).Error
	if err != nil {
		return rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	return
}

func (ds *DeptService) GetDepts() (topDepts []model.Dept, err error) {
	treeMap, err := ds.getDeptTree()
	if err != nil {
		return topDepts, err
	}
	topDepts = treeMap[0]
	for i := 0; i < len(topDepts); i++ {
		err = ds.getChildrenList(&topDepts[i], treeMap)
	}
	return
}

func (ds *DeptService) getChildrenList(dept *model.Dept, treeMap map[int][]model.Dept) (err error) {
	dept.Children = treeMap[dept.DeptID]
	for i := 0; i < len(dept.Children); i++ {
		err = ds.getChildrenList(&dept.Children[i], treeMap)
	}
	return err
}

func (ds *DeptService) getDeptTree() (treeMap map[int][]model.Dept, err error) {
	var allDepts []model.Dept
	treeMap = make(map[int][]model.Dept)
	err = global.DB.Order("sort").Find(&allDepts).Error
	if err != nil {
		return treeMap, rescode.RegisterErrorWithCause(err, code.ErrCodeMap[code.ErrDatabaseCode])
	}
	for _, v := range allDepts {
		treeMap[v.ParentId] = append(treeMap[v.ParentId], v)
	}
	return
}
